package com.kakao.sdk.navi.sample;

import android.Manifest;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.SearchView;
import android.util.Log;
import android.view.MenuItem;
import android.widget.Toast;

import com.kakao.sdk.navi.sample.daummap.Item;
import com.kakao.sdk.navi.sample.daummap.OnFinishSearchListener;
import com.kakao.sdk.navi.sample.daummap.Searcher;

import net.daum.mf.map.api.MapPOIItem;
import net.daum.mf.map.api.MapPoint;
import net.daum.mf.map.api.MapView;

import java.util.List;

public class KakaoNaviMapActivity extends AppCompatActivity implements MapView.MapViewEventListener, SearchView.OnQueryTextListener, MapView.CurrentLocationEventListener, MapView.POIItemEventListener, LocationListener {
    private MapView mapView;
    private SearchView searchView;
    private LocationManager locationManager;
    private final String API_KEY = "DAUM_MAP_KAKAO_NAVI_ANDROID_DEMO_APIKEY";
    private static final String[] LOCATION_PERMS = {
            Manifest.permission.ACCESS_COARSE_LOCATION,
            Manifest.permission.ACCESS_FINE_LOCATION
    };
    private static final int LOCATION_REQUEST = 1340;

    private static final int SHARE_REQ_CODE = 1;
    private static final int NAVI_REQ_CODE = 2;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_kakaonavi_map);

        if (getSupportActionBar() != null)
            getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        mapView = (MapView) findViewById(R.id.map_view);
        mapView.setDaumMapApiKey(API_KEY);

        mapView.setMapViewEventListener(this);

        searchView = (SearchView) findViewById(R.id.location_search_view);
        searchView.setOnQueryTextListener(this);

        locationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                // this takes the user 'back', as if they pressed the left-facing triangle icon on the main android toolbar.
                // if this doesn't work as desired, another possibility is to call `finish()` here.
                onBackPressed();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    @Override
    public boolean onQueryTextSubmit(String query) {
        updateMap(query);
        return false;
    }

    @Override
    public boolean onQueryTextChange(String newText) {
        return false;
    }

    private void initMapListeners() {
//        if (ContextCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED) {
//            locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 5000, 10, this);
//            Location location = locationManager.getLastKnownLocation(LocationManager.GPS_PROVIDER);
//
//            if (location != null) {
//                mapView.setMapCenterPoint(MapPoint.mapPointWithGeoCoord(location.getLatitude(), location.getLongitude()), true);
//                Log.e(this.getClass().getName(), location.toString());
//            }
//        }
        mapView.setPOIItemEventListener(this);
    }

//    private boolean checkPermissionAboveM() {
//        if (ContextCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED) {
//            return true;
//        }
//        ActivityCompat.requestPermissions(this, LOCATION_PERMS, LOCATION_REQUEST);
//        return false;
//    }

//    @Override
//    public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
//        if (requestCode == LOCATION_REQUEST) {
//            if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
//                Log.e(this.getClass().getName(), "Successfully acquired ACCESS_FINE_LOCATION permission.");
//                initMapListeners();
//            } else {
//                Log.e(this.getClass().getName(), "Failed to get location permission.");
//                Toast.makeText(getApplicationContext(), "Failed to get location permission.", Toast.LENGTH_LONG).show();
//            }
//        }
//    }

    private void updateMap(String locationQuery) {
        if (locationQuery == null || locationQuery.length() < 2) {
            Toast.makeText(getApplicationContext(), getResources().getString(R.string.text_enter_location), Toast.LENGTH_LONG).show();
            return;
        }

        MapPoint.GeoCoordinate geoCoordinate = mapView.getMapCenterPoint().getMapPointGeoCoord();
        Searcher searcher = new Searcher();
        searcher.searchKeyword(getApplicationContext(), locationQuery, geoCoordinate.latitude, geoCoordinate.longitude, 10000, 1, API_KEY, new OnFinishSearchListener() {
            @Override
            public void onSuccess(List<Item> itemList) {
                mapView.removeAllPOIItems();
                drawItemsOnMap(itemList);
            }

            @Override
            public void onFail() {
            }
        });
    }

    public void drawItemsOnMap(List<Item> items) {
        if (items.size() == 0) return;;
        Item firstItem = items.get(0);
        mapView.setMapCenterPoint(MapPoint.mapPointWithGeoCoord(firstItem.latitude, firstItem.longitude), true);

        for (Item item : items) {
            MapPOIItem marker = new MapPOIItem();
            marker.setItemName(item.title);
            marker.setTag(0);
            marker.setMapPoint(MapPoint.mapPointWithGeoCoord(item.latitude, item.longitude));
            marker.setMarkerType(MapPOIItem.MarkerType.BluePin);
            marker.setSelectedMarkerType(MapPOIItem.MarkerType.RedPin);
            mapView.addPOIItem(marker);
        }
    }

    @Override
    public void onLocationChanged(Location location) {

    }

    @Override
    public void onStatusChanged(String provider, int status, Bundle extras) {

    }

    @Override
    public void onProviderEnabled(String provider) {

    }

    @Override
    public void onProviderDisabled(String provider) {

    }

    @Override
    public void onCurrentLocationUpdate(MapView mapView, MapPoint mapPoint, float v) {

    }

    @Override
    public void onCurrentLocationDeviceHeadingUpdate(MapView mapView, float v) {

    }

    @Override
    public void onCurrentLocationUpdateFailed(MapView mapView) {

    }

    @Override
    public void onCurrentLocationUpdateCancelled(MapView mapView) {

    }

    @Override
    public void onMapViewInitialized(MapView mapView) {
//        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
//            if (!checkPermissionAboveM()) {
//                return;
//            }
//        }
        initMapListeners();
    }

    @Override
    public void onMapViewCenterPointMoved(MapView mapView, MapPoint mapPoint) {

    }

    @Override
    public void onMapViewZoomLevelChanged(MapView mapView, int i) {

    }

    @Override
    public void onMapViewSingleTapped(MapView mapView, MapPoint mapPoint) {

    }

    @Override
    public void onMapViewDoubleTapped(MapView mapView, MapPoint mapPoint) {

    }

    @Override
    public void onMapViewLongPressed(MapView mapView, MapPoint mapPoint) {

    }

    @Override
    public void onMapViewDragStarted(MapView mapView, MapPoint mapPoint) {

    }

    @Override
    public void onMapViewDragEnded(MapView mapView, MapPoint mapPoint) {

    }

    @Override
    public void onMapViewMoveFinished(MapView mapView, MapPoint mapPoint) {

    }

    @Override
    public void onPOIItemSelected(MapView mapView, final MapPOIItem mapPOIItem) {
        new AlertDialog.Builder(this).setTitle(mapPOIItem.getItemName()).setMessage(getResources().getString(R.string.text_choose_location)).setPositiveButton(getResources().getString(R.string.ok), new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                Intent intent = new Intent();
                intent.putExtra("name", mapPOIItem.getItemName());
                intent.putExtra("x", mapPOIItem.getMapPoint().getMapPointGeoCoord().longitude);
                intent.putExtra("y", mapPOIItem.getMapPoint().getMapPointGeoCoord().latitude);
                setResult(RESULT_OK, intent);
                finish();
            }
        }).setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {

            }
        }).create().show();
    }

    @Override
    public void onCalloutBalloonOfPOIItemTouched(MapView mapView, MapPOIItem mapPOIItem) {

    }

    @Override
    public void onCalloutBalloonOfPOIItemTouched(MapView mapView, MapPOIItem mapPOIItem, MapPOIItem.CalloutBalloonButtonType calloutBalloonButtonType) {

    }

    @Override
    public void onDraggablePOIItemMoved(MapView mapView, MapPOIItem mapPOIItem, MapPoint mapPoint) {

    }
}
