package com.kakao.s2;

import android.net.Uri;
import android.os.Build;

import com.kakao.network.IRequest;
import com.kakao.network.ServerProtocol;
import com.kakao.network.multipart.Part;
import com.kakao.util.exception.KakaoException;
import com.kakao.util.helper.CommonProtocol;
import com.kakao.util.helper.SystemInfo;
import com.kakao.util.helper.Utility;
import com.kakao.util.helper.log.Logger;

import org.json.JSONException;

import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author kevin.kang
 * Created by kevin.kang on 2016. 8. 22..
 */
class EventsLogRequest implements IRequest {
    private static final String POST = "POST";
    private String appKey;
    private Event rootEvent;
    private List<Event> events;

    EventsLogRequest(final String appKey, Event rootEvent, List<Event> leafEvents) {
        this.appKey = appKey;
        this.rootEvent = rootEvent;
        this.events = leafEvents;
    }

    private static String createBaseURL(final String authority, final String requestPath) {
        Uri uri = Utility.buildUri(authority, requestPath);
        return uri.toString();
    }

    @Override
    public String getMethod() {
        return POST;
    }

    @Override
    public String getUrl() {
        return createBaseURL(ServerProtocol.API_AUTHORITY, ServerProtocol.EVENTS_PUBLISH_PATH);
    }


    @Override
    public Map<String, String> getParams() {
        Map<String, String> params = new HashMap<String, String>();

        if (events != null && !events.isEmpty()) {
            String eventsString = events.toString();
            if (eventsString.length() > Event.MAX_BODY_SIZE) {
                throw new KakaoException(KakaoException.ErrorType.ILLEGAL_ARGUMENT, "Event's bulk size is too large (over " + Event.MAX_BODY_SIZE + " bytes).");
            }
            params.put(Event.EVENTS, eventsString);
        }

        if (rootEvent == null) {
            return params;
        }

        if (rootEvent.getTimestamp() == 0) {
            params.put(Event.TIMESTAMP, String.valueOf(rootEvent.getTimestamp()));
        } else {
            params.put(Event.TIMESTAMP, String.valueOf(new Date().getTime()));
        }
        if (rootEvent.getFrom() != null)
            params.put(Event.FROM, rootEvent.getFrom());
        if (rootEvent.getTo() != null)
            params.put(Event.TO, rootEvent.getTo());
        if (rootEvent.getAction() != null)
            params.put(Event.ACTION, rootEvent.getAction());
        if (!rootEvent.getProps().isEmpty()) {
            try {
                params.put(Event.PROPS, this.rootEvent.propsToJson().toString());
            } catch (JSONException e) {
                Logger.e("failed to jsonify properties for event: " + e.toString());
            }
        }
        return params;
    }

    @Override
    public Map<String, String> getHeaders() {
        Map<String, String> header = new HashMap<String, String>();
        header.put(CommonProtocol.KA_HEADER_KEY, SystemInfo.getKAHeader());
        if (!header.containsKey("Content-Type")) {
            header.put("Content-Type", "application/x-www-form-urlencoded");
        }

        if (!header.containsKey("Accept")) {
            header.put("Accept", "*/*");
        }

        if (!header.containsKey("User-Agent")) {
            header.put("User-Agent", getHttpUserAgentString());
        }
        header.put(CommonProtocol.KA_AUTH_HEADER_KEY, CommonProtocol.KAKAO_AK_HEADER_KEY + " " + appKey);
        return header;
    }

    @Override
    public List<Part> getMultiPartList() {
        return Collections.emptyList();
    }

    @Override
    public String getBodyEncoding() {
        return "UTF-8";
    }

    private String getHttpUserAgentString() {
        return new StringBuilder().append("os/").append(CommonProtocol.OS_ANDROID).append("-").append(Build.VERSION.SDK_INT).append(" ").toString();
    }
}
